@@ -70,7 +70,7 @@ BUILD_LOGGER_APPEND_GLOBAL_LOGS_ENDPOINT = "/build/%(build_id)s"
 BUILD_LOGGER_CREATE_TEST_ENDPOINT = "/build/%(build_id)s/test"
 BUILD_LOGGER_APPEND_TEST_LOGS_ENDPOINT = "/build/%(build_id)s/test/%(test_id)s"
 
-BUILD_LOGGER_DEFAULT_URL = "https://logkeeper.mongodb.org"
+BUILD_LOGGER_DEFAULT_URL = "https://logkeeper2.build.10gen.cc"
 BUILD_LOGGER_TIMEOUT_SECS = 65
 
 LOCAL_REPORTS_DIR = "bd_reports"
@@ -86,7 +86,7 @@ THIRD_PARTY_COMPONENTS_FILE = "etc/third_party_components.yml"
 
 ############################################################################
 
-RE_LETTERS = re.compile("[A-Za-z]{2,}")
+RE_LETTER = re.compile("[A-Za-z]")
 
 
 def default_if_none(value, default):
@@ -287,7 +287,9 @@ class VersionInfo:
             self.production_version = True
 
             # Abseil has an empty string for one version
-            if self.ver_str == "":
+            # Abseil thinks "~" is character to use in versions
+            # Geary thinks "+" is character to use in versions
+            if self.ver_str == "" or "~" in self.ver_str or "+" in self.ver_str:
                 self.production_version = False
                 return
 
@@ -329,8 +331,9 @@ class VersionInfo:
             if self.ver_str.endswith('-'):
                 self.ver_str = self.ver_str[0:-1]
 
-            # Boost keeps varying the version strings so filter for anything with 2 or more ascii charaters
-            if RE_LETTERS.search(self.ver_str):
+            # Boost keeps varying the version strings so filter for anything with a letter
+            # Safeint has "3.0.26c" as a version number
+            if RE_LETTER.search(self.ver_str):
                 self.production_version = False
                 return
 
@@ -422,6 +425,23 @@ def _test_version_info():
     assert VersionInfo('7.0.2.2') <= VersionInfo('8.0.2')
 
 
+def _retry_on_except(count, func):
+    # Retry func() COUNT times until func() does not raise an exception
+    # pylint: disable=bare-except
+    retry = 0
+    while retry < count:
+
+        try:
+            return func()
+        except:
+            exception_info = sys.exc_info()[0]
+            LOGGER.error("Failed to execute retriable function (%s), retrying", exception_info)
+
+        retry += 1
+
+    raise ValueError("Failed to run query after retries %s" % (count))
+
+
 class Component:
     """
     Black Duck Component description.
@@ -469,7 +489,17 @@ class Component:
             versions_url = component["component"] + f"/versions?sort=releasedon%20desc&limit={limit}"
 
             LOGGER.info("Retrieving version information via %s", versions_url)
-            vjson = hub.execute_get(versions_url).json()
+
+            def get_version_info():
+                vjson = hub.execute_get(versions_url).json()
+
+                if "items" not in vjson:
+                    LOGGER.warn("Missing items in response: %s", vjson)
+                    raise ValueError("Missing items in response for " + versions_url)
+
+                return vjson
+
+            vjson = _retry_on_except(5, get_version_info)
 
             versions = [(ver["versionName"], ver["releasedOn"]) for ver in vjson["items"]]
 
@@ -543,7 +573,7 @@ curl --retry 5 -s -L https://detect.synopsys.com/detect.sh  | bash -s -- --black
 """.encode())
         fp.flush()
 
-        subprocess.call(["/bin/sh", fp.name])
+        subprocess.check_call(["/bin/sh", fp.name])
 
 
 def _scan_cmd_args(args):
@@ -559,6 +589,15 @@ def _query_blackduck():
 
     hub = HubInstance()
 
+    LOGGER.info("Getting version from blackduck")
+    version = hub.execute_get(hub.get_urlbase() + "/api/current-version").json()
+    LOGGER.info("Version: %s", version)
+
+    # Get a list of all projects, this is a privileged call and will fail if we do not have a valid license
+    LOGGER.info("Get All Projects")
+    projects = hub.get_projects()
+    LOGGER.info("Projects: %s", projects)
+
     LOGGER.info("Fetching project %s from blackduck", BLACKDUCK_PROJECT)
     project = hub.get_project_by_name(BLACKDUCK_PROJECT)
 
