@@ -26,21 +26,20 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-
-/**
- * This file contains tests for mongo/db/commands/mr.h
- */
-
-#include "mongo/db/commands/mr.h"
-
 #include <functional>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/commands/map_reduce_gen.h"
+#include "mongo/db/commands/mr_common.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/json.h"
 #include "mongo/db/op_observer_noop.h"
@@ -59,11 +58,7 @@ namespace mongo {
 namespace {
 
 /**
- * Tests for mr::Config
- */
-
-/**
- * Helper function to verify field of mr::Config::OutputOptions.
+ * Helper function to verify field of map_reduce_common::OutputOptions.
  */
 template <typename T>
 void _compareOutputOptionField(const std::string& dbname,
@@ -78,17 +73,17 @@ void _compareOutputOptionField(const std::string& dbname,
 }
 
 /**
- * Returns string representation of mr::Config::OutputType
+ * Returns string representation of OutputType
  */
-std::string _getOutTypeString(mr::Config::OutputType outType) {
+std::string _getOutTypeString(OutputType outType) {
     switch (outType) {
-        case mr::Config::REPLACE:
+        case OutputType::Replace:
             return "REPLACE";
-        case mr::Config::MERGE:
+        case OutputType::Merge:
             return "MERGE";
-        case mr::Config::REDUCE:
+        case OutputType::Reduce:
             return "REDUCE";
-        case mr::Config::INMEMORY:
+        case OutputType::InMemory:
             return "INMEMORY";
     }
     MONGO_UNREACHABLE;
@@ -103,9 +98,10 @@ void _testConfigParseOutputOptions(const std::string& dbname,
                                    const std::string& expectedCollectionName,
                                    const std::string& expectedFinalNamespace,
                                    bool expectedOutNonAtomic,
-                                   mr::Config::OutputType expectedOutType) {
+                                   OutputType expectedOutType) {
     const BSONObj cmdObj = fromjson(cmdObjStr);
-    mr::Config::OutputOptions outputOptions = mr::Config::parseOutputOptions(dbname, cmdObj);
+    map_reduce_common::OutputOptions outputOptions =
+        map_reduce_common::parseOutputOptions(dbname, cmdObj);
     _compareOutputOptionField(dbname, cmdObjStr, "outDb", outputOptions.outDB, expectedOutDb);
     _compareOutputOptionField(
         dbname, cmdObjStr, "collectionName", outputOptions.collectionName, expectedCollectionName);
@@ -124,49 +120,50 @@ void _testConfigParseOutputOptions(const std::string& dbname,
 }
 
 /**
- * Tests for mr::Config::parseOutputOptions.
+ * Tests for map_reduce_common::parseOutputOptions.
  */
 TEST(ConfigOutputOptionsTest, parseOutputOptions) {
     // Missing 'out' field.
-    ASSERT_THROWS(mr::Config::parseOutputOptions("mydb", fromjson("{}")), AssertionException);
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions("mydb", fromjson("{}")),
+                  AssertionException);
     // 'out' must be either string or object.
-    ASSERT_THROWS(mr::Config::parseOutputOptions("mydb", fromjson("{out: 99}")),
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions("mydb", fromjson("{out: 99}")),
                   AssertionException);
-    // 'out.nonAtomic' is not supported with normal, replace or inline.
-    ASSERT_THROWS(mr::Config::parseOutputOptions(
-                      "mydb", fromjson("{out: {normal: 'mycoll', nonAtomic: true}}")),
+    // 'out.nonAtomic' false is not supported.
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(
+                      "mydb", fromjson("{out: {normal: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
-    ASSERT_THROWS(mr::Config::parseOutputOptions(
-                      "mydb", fromjson("{out: {replace: 'mycoll', nonAtomic: true}}")),
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(
+                      "mydb", fromjson("{out: {replace: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
-    ASSERT_THROWS(mr::Config::parseOutputOptions(
-                      "mydb", fromjson("{out: {inline: 'mycoll', nonAtomic: true}}")),
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(
+                      "mydb", fromjson("{out: {inline: 'mycoll', nonAtomic: false}}")),
                   AssertionException);
-    // Unknown output specifer.
-    ASSERT_THROWS(
-        mr::Config::parseOutputOptions("mydb", fromjson("{out: {no_such_out_type: 'mycoll'}}")),
-        AssertionException);
 
+    // Unknown output specifer.
+    ASSERT_THROWS(map_reduce_common::parseOutputOptions(
+                      "mydb", fromjson("{out: {no_such_out_type: 'mycoll'}}")),
+                  AssertionException);
 
     // 'out' is string.
     _testConfigParseOutputOptions(
-        "mydb", "{out: 'mycoll'}", "", "mycoll", "mydb.mycoll", false, mr::Config::REPLACE);
+        "mydb", "{out: 'mycoll'}", "", "mycoll", "mydb.mycoll", true, OutputType::Replace);
     // 'out' is object.
     _testConfigParseOutputOptions("mydb",
                                   "{out: {normal: 'mycoll'}}",
                                   "",
                                   "mycoll",
                                   "mydb.mycoll",
-                                  false,
-                                  mr::Config::REPLACE);
+                                  true,
+                                  OutputType::Replace);
     // 'out.db' overrides dbname parameter
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {replace: 'mycoll', db: 'mydb2'}}",
                                   "mydb2",
                                   "mycoll",
                                   "mydb2.mycoll",
-                                  false,
-                                  mr::Config::REPLACE);
+                                  true,
+                                  OutputType::Replace);
     // 'out.nonAtomic' is supported with merge and reduce.
     _testConfigParseOutputOptions("mydb",
                                   "{out: {merge: 'mycoll', nonAtomic: true}}",
@@ -174,22 +171,22 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   "mycoll",
                                   "mydb.mycoll",
                                   true,
-                                  mr::Config::MERGE);
+                                  OutputType::Merge);
     _testConfigParseOutputOptions("mydb",
                                   "{out: {reduce: 'mycoll', nonAtomic: true}}",
                                   "",
                                   "mycoll",
                                   "mydb.mycoll",
                                   true,
-                                  mr::Config::REDUCE);
+                                  OutputType::Reduce);
     // inline
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {inline: 'mycoll', db: 'mydb2'}}",
                                   "mydb2",
                                   "",
                                   "",
-                                  false,
-                                  mr::Config::INMEMORY);
+                                  true,
+                                  OutputType::InMemory);
 
     // Order should not matter in fields of 'out' object.
     _testConfigParseOutputOptions("mydb1",
@@ -197,75 +194,36 @@ TEST(ConfigOutputOptionsTest, parseOutputOptions) {
                                   "mydb2",
                                   "mycoll",
                                   "mydb2.mycoll",
-                                  false,
-                                  mr::Config::REPLACE);
+                                  true,
+                                  OutputType::Replace);
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {db: 'mydb2', replace: 'mycoll'}}",
                                   "mydb2",
                                   "mycoll",
                                   "mydb2.mycoll",
-                                  false,
-                                  mr::Config::REPLACE);
+                                  true,
+                                  OutputType::Replace);
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {nonAtomic: true, merge: 'mycoll'}}",
                                   "",
                                   "mycoll",
                                   "mydb1.mycoll",
                                   true,
-                                  mr::Config::MERGE);
+                                  OutputType::Merge);
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {nonAtomic: true, reduce: 'mycoll'}}",
                                   "",
                                   "mycoll",
                                   "mydb1.mycoll",
                                   true,
-                                  mr::Config::REDUCE);
+                                  OutputType::Reduce);
     _testConfigParseOutputOptions("mydb1",
                                   "{out: {db: 'mydb2', inline: 'mycoll'}}",
                                   "mydb2",
                                   "",
                                   "",
-                                  false,
-                                  mr::Config::INMEMORY);
-}
-
-TEST(ConfigTest, ParseCollation) {
-    std::string dbname = "myDB";
-    BSONObj collation = BSON("locale"
-                             << "en_US");
-    BSONObjBuilder bob;
-    bob.append("mapReduce", "myCollection");
-    bob.appendCode("map", "function() { emit(0, 1); }");
-    bob.appendCode("reduce", "function(k, v) { return {count: 0}; }");
-    bob.append("out", "outCollection");
-    bob.append("collation", collation);
-    BSONObj cmdObj = bob.obj();
-    mr::Config config(dbname, cmdObj);
-    ASSERT_BSONOBJ_EQ(config.collation, collation);
-}
-
-TEST(ConfigTest, ParseNoCollation) {
-    std::string dbname = "myDB";
-    BSONObjBuilder bob;
-    bob.append("mapReduce", "myCollection");
-    bob.appendCode("map", "function() { emit(0, 1); }");
-    bob.appendCode("reduce", "function(k, v) { return {count: 0}; }");
-    bob.append("out", "outCollection");
-    BSONObj cmdObj = bob.obj();
-    mr::Config config(dbname, cmdObj);
-    ASSERT_BSONOBJ_EQ(config.collation, BSONObj());
-}
-
-TEST(ConfigTest, CollationNotAnObjectFailsToParse) {
-    std::string dbname = "myDB";
-    BSONObjBuilder bob;
-    bob.append("mapReduce", "myCollection");
-    bob.appendCode("map", "function() { emit(0, 1); }");
-    bob.appendCode("reduce", "function(k, v) { return {count: 0}; }");
-    bob.append("out", "outCollection");
-    bob.append("collation", "en_US");
-    BSONObj cmdObj = bob.obj();
-    ASSERT_THROWS(mr::Config(dbname, cmdObj), AssertionException);
+                                  true,
+                                  OutputType::InMemory);
 }
 
 /**
@@ -273,6 +231,27 @@ TEST(ConfigTest, CollationNotAnObjectFailsToParse) {
  */
 class MapReduceOpObserver : public OpObserverNoop {
 public:
+    /**
+     * This function is called whenever mapReduce copies indexes from an existing output collection
+     * to a temporary collection.
+     */
+    void onCreateIndex(OperationContext* opCtx,
+                       const NamespaceString& nss,
+                       CollectionUUID uuid,
+                       BSONObj indexDoc,
+                       bool fromMigrate) override;
+
+    /**
+     * This function is called whenever mapReduce copies indexes from an existing output collection
+     * to a temporary collection.
+     */
+    void onStartIndexBuild(OperationContext* opCtx,
+                           const NamespaceString& nss,
+                           CollectionUUID collUUID,
+                           const UUID& indexBuildUUID,
+                           const std::vector<BSONObj>& indexes,
+                           bool fromMigrate) override;
+
     /**
      * This function is called whenever mapReduce inserts documents into a temporary output
      * collection.
@@ -288,12 +267,13 @@ public:
      * Tracks the temporary collections mapReduces creates.
      */
     void onCreateCollection(OperationContext* opCtx,
-                            Collection* coll,
+                            const CollectionPtr& coll,
                             const NamespaceString& collectionName,
                             const CollectionOptions& options,
                             const BSONObj& idIndex,
                             const OplogSlot& createOpTime) override;
 
+    using OpObserver::onDropCollection;
     repl::OpTime onDropCollection(OperationContext* opCtx,
                                   const NamespaceString& collectionName,
                                   OptionalCollectionUUID uuid,
@@ -304,12 +284,34 @@ public:
     // while mapReduce inserts its results into the temporary output collection.
     std::function<void()> onInsertsFn = [] {};
 
+    // Holds indexes copied from existing output collection to the temporary collections.
+    std::vector<BSONObj> indexesCreated;
+
     // Holds namespaces of temporary collections created by mapReduce.
     std::vector<NamespaceString> tempNamespaces;
 
     const repl::OpTime dropOpTime = {Timestamp(Seconds(100), 1U), 1LL};
 };
 
+void MapReduceOpObserver::onCreateIndex(OperationContext* opCtx,
+                                        const NamespaceString& nss,
+                                        CollectionUUID uuid,
+                                        BSONObj indexDoc,
+                                        bool fromMigrate) {
+    indexesCreated.push_back(indexDoc.getOwned());
+}
+
+void MapReduceOpObserver::onStartIndexBuild(OperationContext* opCtx,
+                                            const NamespaceString& nss,
+                                            CollectionUUID collUUID,
+                                            const UUID& indexBuildUUID,
+                                            const std::vector<BSONObj>& indexes,
+                                            bool fromMigrate) {
+    for (auto&& obj : indexes) {
+        indexesCreated.push_back(obj.getOwned());
+    }
+}
+
 void MapReduceOpObserver::onInserts(OperationContext* opCtx,
                                     const NamespaceString& nss,
                                     OptionalCollectionUUID uuid,
@@ -320,7 +322,7 @@ void MapReduceOpObserver::onInserts(OperationContext* opCtx,
 }
 
 void MapReduceOpObserver::onCreateCollection(OperationContext*,
-                                             Collection*,
+                                             const CollectionPtr&,
                                              const NamespaceString& collectionName,
                                              const CollectionOptions& options,
                                              const BSONObj&,
@@ -401,7 +403,7 @@ void MapReduceCommandTest::setUp() {
 
     repl::DropPendingCollectionReaper::set(
         service,
-        stdx::make_unique<repl::DropPendingCollectionReaper>(repl::StorageInterface::get(service)));
+        std::make_unique<repl::DropPendingCollectionReaper>(repl::StorageInterface::get(service)));
 
     // Set up an OpObserver to track the temporary collections mapReduce creates.
     auto opObserver = std::make_unique<MapReduceOpObserver>();
@@ -413,6 +415,7 @@ void MapReduceCommandTest::setUp() {
 
     // Transition to PRIMARY so that the server can accept writes.
     ASSERT_OK(_getReplCoord()->setFollowerMode(repl::MemberState::RS_PRIMARY));
+    repl::createOplog(_opCtx.get());
 
     // Create collection with one document.
     CollectionOptions collectionOptions;
@@ -517,5 +520,48 @@ TEST_F(MapReduceCommandTest, PrimaryStepDownPreventsTemporaryCollectionDrops) {
     }
 }
 
+TEST_F(MapReduceCommandTest, ReplacingExistingOutputCollectionPreservesIndexes) {
+    CollectionOptions options;
+    options.uuid = UUID::gen();
+    ASSERT_OK(_storage.createCollection(_opCtx.get(), outputNss, options));
+
+    // Create an index in the existing output collection. This should be present in the recreated
+    // output collection.
+    auto indexSpec = BSON("v" << 2 << "key" << BSON("a" << 1) << "name"
+                              << "a_1");
+    {
+        AutoGetCollection coll(_opCtx.get(), outputNss, MODE_X);
+        ASSERT(coll);
+        writeConflictRetry(
+            _opCtx.get(), "ReplacingExistingOutputCollectionPreservesIndexes", outputNss.ns(), [&] {
+                WriteUnitOfWork wuow(_opCtx.get());
+                ASSERT_OK(
+                    coll.getWritableCollection()->getIndexCatalog()->createIndexOnEmptyCollection(
+                        _opCtx.get(), coll.getWritableCollection(), indexSpec));
+                wuow.commit();
+            });
+    }
+
+    auto sourceDoc = BSON("_id" << 0);
+    ASSERT_OK(_storage.insertDocument(_opCtx.get(), inputNss, {sourceDoc, Timestamp(0)}, 1LL));
+
+    auto mapCode = "function() { emit(this._id, this._id); }"_sd;
+    auto reduceCode = "function(k, v) { return Array.sum(v); }"_sd;
+    ASSERT_OK(_runCommand(mapCode, reduceCode));
+
+    // MapReduce should filter existing indexes in the temporary collection, such as
+    // the _id index.
+    ASSERT_EQUALS(1U, _opObserver->indexesCreated.size())
+        << BSON("indexesCreated" << _opObserver->indexesCreated);
+    ASSERT_BSONOBJ_EQ(indexSpec, _opObserver->indexesCreated[0]);
+
+    ASSERT_NOT_EQUALS(
+        *options.uuid,
+        *CollectionCatalog::get(_opCtx.get())->lookupUUIDByNSS(_opCtx.get(), outputNss))
+        << "Output collection " << outputNss << " was not replaced";
+
+    _assertTemporaryCollectionsAreDropped();
+}
+
 }  // namespace
 }  // namespace mongo
