@@ -29,17 +29,14 @@
 
 #include "mongo/platform/basic.h"
 
-#include <memory>
-#include <set>
-#include <string>
-
 #include "mongo/db/jsobj.h"
+#include "mongo/db/keys_collection_client_direct.h"
 #include "mongo/db/keys_collection_client_sharded.h"
-#include "mongo/db/keys_collection_document.h"
+#include "mongo/db/keys_collection_document_gen.h"
 #include "mongo/db/keys_collection_manager.h"
-#include "mongo/db/logical_clock.h"
-#include "mongo/s/catalog/dist_lock_manager_mock.h"
-#include "mongo/s/config_server_test_fixture.h"
+#include "mongo/db/s/config/config_server_test_fixture.h"
+#include "mongo/db/time_proof_service.h"
+#include "mongo/db/vector_clock_mutable.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/clock_source_mock.h"
 #include "mongo/util/fail_point.h"
@@ -75,15 +72,6 @@ protected:
         ConfigServerTestFixture::tearDown();
     }
 
-    /**
-     * Intentionally create a DistLockManagerMock, even though this is a config serfver test in
-     * order to avoid the lock pinger thread from executing and accessing uninitialized state.
-     */
-    std::unique_ptr<DistLockManager> makeDistLockManager(
-        std::unique_ptr<DistLockCatalog> distLockCatalog) override {
-        return std::make_unique<DistLockManagerMock>(std::move(distLockCatalog));
-    }
-
 private:
     std::unique_ptr<KeysCollectionManager> _keyManager;
 };
@@ -93,7 +81,7 @@ TEST_F(KeysManagerShardedTest, GetKeyForValidationTimesOutIfRefresherIsNotRunnin
                                               ErrorCodes::ExceededTimeLimit);
 
     ASSERT_THROWS(
-        keyManager()->getKeyForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0))),
+        keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0))),
         DBException);
 }
 
@@ -101,23 +89,24 @@ TEST_F(KeysManagerShardedTest, GetKeyForValidationErrorsIfKeyDoesntExist) {
     keyManager()->startMonitoring(getServiceContext());
 
     auto keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
     ASSERT_EQ(ErrorCodes::KeyNotFound, keyStatus.getStatus());
 }
 
 TEST_F(KeysManagerShardedTest, GetKeyWithSingleKey) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
 
     auto keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
     ASSERT_OK(keyStatus.getStatus());
 
-    auto key = keyStatus.getValue();
+    auto key = keyStatus.getValue().front();
     ASSERT_EQ(1, key.getKeyId());
     ASSERT_EQ(origKey1.getKey(), key.getKey());
     ASSERT_EQ(Timestamp(105, 0), key.getExpiresAt().asTimestamp());
@@ -126,30 +115,32 @@ TEST_F(KeysManagerShardedTest, GetKeyWithSingleKey) {
 TEST_F(KeysManagerShardedTest, GetKeyWithMultipleKeys) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
 
-    KeysCollectionDocument origKey2(
-        2, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(205, 0)));
+    KeysCollectionDocument origKey2(2);
+    origKey2.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(205, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey2.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey2.toBSON()));
 
     auto keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
     ASSERT_OK(keyStatus.getStatus());
 
-    auto key = keyStatus.getValue();
+    auto key = keyStatus.getValue().front();
     ASSERT_EQ(1, key.getKeyId());
     ASSERT_EQ(origKey1.getKey(), key.getKey());
     ASSERT_EQ(Timestamp(105, 0), key.getExpiresAt().asTimestamp());
 
     keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 2, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 2, LogicalTime(Timestamp(100, 0)));
     ASSERT_OK(keyStatus.getStatus());
 
-    key = keyStatus.getValue();
+    key = keyStatus.getValue().front();
     ASSERT_EQ(2, key.getKeyId());
     ASSERT_EQ(origKey2.getKey(), key.getKey());
     ASSERT_EQ(Timestamp(205, 0), key.getExpiresAt().asTimestamp());
@@ -158,45 +149,48 @@ TEST_F(KeysManagerShardedTest, GetKeyWithMultipleKeys) {
 TEST_F(KeysManagerShardedTest, GetKeyShouldErrorIfKeyIdMismatchKey) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
 
     auto keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 2, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 2, LogicalTime(Timestamp(100, 0)));
     ASSERT_EQ(ErrorCodes::KeyNotFound, keyStatus.getStatus());
 }
 
 TEST_F(KeysManagerShardedTest, GetKeyWithoutRefreshShouldReturnRightKey) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
-    KeysCollectionDocument origKey2(
-        2, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(110, 0)));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
+    KeysCollectionDocument origKey2(2);
+    origKey2.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(110, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey2.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey2.toBSON()));
 
     {
-        auto keyStatus = keyManager()->getKeyForValidation(
+        auto keyStatus = keyManager()->getKeysForValidation(
             operationContext(), 1, LogicalTime(Timestamp(100, 0)));
         ASSERT_OK(keyStatus.getStatus());
 
-        auto key = keyStatus.getValue();
+        auto key = keyStatus.getValue().front();
         ASSERT_EQ(1, key.getKeyId());
         ASSERT_EQ(origKey1.getKey(), key.getKey());
         ASSERT_EQ(Timestamp(105, 0), key.getExpiresAt().asTimestamp());
     }
 
     {
-        auto keyStatus = keyManager()->getKeyForValidation(
+        auto keyStatus = keyManager()->getKeysForValidation(
             operationContext(), 2, LogicalTime(Timestamp(105, 0)));
         ASSERT_OK(keyStatus.getStatus());
 
-        auto key = keyStatus.getValue();
+        auto key = keyStatus.getValue().front();
         ASSERT_EQ(2, key.getKeyId());
         ASSERT_EQ(origKey2.getKey(), key.getKey());
         ASSERT_EQ(Timestamp(110, 0), key.getExpiresAt().asTimestamp());
@@ -206,10 +200,11 @@ TEST_F(KeysManagerShardedTest, GetKeyWithoutRefreshShouldReturnRightKey) {
 TEST_F(KeysManagerShardedTest, GetKeyForSigningShouldReturnRightKey) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
 
     keyManager()->refreshNow(operationContext());
 
@@ -225,14 +220,16 @@ TEST_F(KeysManagerShardedTest, GetKeyForSigningShouldReturnRightKey) {
 TEST_F(KeysManagerShardedTest, GetKeyForSigningShouldReturnRightOldKey) {
     keyManager()->startMonitoring(getServiceContext());
 
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
-    KeysCollectionDocument origKey2(
-        2, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(110, 0)));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
+    KeysCollectionDocument origKey2(2);
+    origKey2.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(110, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey2.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey2.toBSON()));
 
     keyManager()->refreshNow(operationContext());
 
@@ -261,7 +258,7 @@ TEST_F(KeysManagerShardedTest, ShouldCreateKeysIfKeyGeneratorEnabled) {
     keyManager()->startMonitoring(getServiceContext());
 
     const LogicalTime currentTime(LogicalTime(Timestamp(100, 0)));
-    LogicalClock::get(operationContext())->setClusterTimeFromTrustedSource(currentTime);
+    VectorClockMutable::get(operationContext())->tickClusterTimeTo(currentTime);
 
     keyManager()->enableKeyGenerator(operationContext(), true);
     keyManager()->refreshNow(operationContext());
@@ -277,7 +274,7 @@ TEST_F(KeysManagerShardedTest, EnableModeFlipFlopStressTest) {
     keyManager()->startMonitoring(getServiceContext());
 
     const LogicalTime currentTime(LogicalTime(Timestamp(100, 0)));
-    LogicalClock::get(operationContext())->setClusterTimeFromTrustedSource(currentTime);
+    VectorClockMutable::get(operationContext())->tickClusterTimeTo(currentTime);
 
     bool doEnable = true;
 
@@ -296,14 +293,15 @@ TEST_F(KeysManagerShardedTest, EnableModeFlipFlopStressTest) {
 }
 
 TEST_F(KeysManagerShardedTest, ShouldStillBeAbleToUpdateCacheEvenIfItCantCreateKeys) {
-    KeysCollectionDocument origKey1(
-        1, "dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0)));
+    KeysCollectionDocument origKey1(1);
+    origKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(105, 0))});
     ASSERT_OK(insertToConfigCollection(
-        operationContext(), KeysCollectionDocument::ConfigNS, origKey1.toBSON()));
+        operationContext(), NamespaceString::kKeysCollectionNamespace, origKey1.toBSON()));
 
     // Set the time to be very ahead so the updater will be forced to create new keys.
     const LogicalTime fakeTime(Timestamp(20000, 0));
-    LogicalClock::get(operationContext())->setClusterTimeFromTrustedSource(fakeTime);
+    VectorClockMutable::get(operationContext())->tickClusterTimeTo(fakeTime);
 
     FailPointEnableBlock failWriteBlock("failCollectionInserts");
 
@@ -314,10 +312,10 @@ TEST_F(KeysManagerShardedTest, ShouldStillBeAbleToUpdateCacheEvenIfItCantCreateK
     }
 
     auto keyStatus =
-        keyManager()->getKeyForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
+        keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(100, 0)));
     ASSERT_OK(keyStatus.getStatus());
 
-    auto key = keyStatus.getValue();
+    auto key = keyStatus.getValue().front();
     ASSERT_EQ(1, key.getKeyId());
     ASSERT_EQ(origKey1.getKey(), key.getKey());
     ASSERT_EQ(Timestamp(105, 0), key.getExpiresAt().asTimestamp());
@@ -325,7 +323,7 @@ TEST_F(KeysManagerShardedTest, ShouldStillBeAbleToUpdateCacheEvenIfItCantCreateK
 
 TEST_F(KeysManagerShardedTest, ShouldNotCreateKeysWithDisableKeyGenerationFailPoint) {
     const LogicalTime currentTime(Timestamp(100, 0));
-    LogicalClock::get(operationContext())->setClusterTimeFromTrustedSource(currentTime);
+    VectorClockMutable::get(operationContext())->tickClusterTimeTo(currentTime);
 
     {
         FailPointEnableBlock failKeyGenerationBlock("disableKeyGeneration");
@@ -333,7 +331,7 @@ TEST_F(KeysManagerShardedTest, ShouldNotCreateKeysWithDisableKeyGenerationFailPo
         keyManager()->enableKeyGenerator(operationContext(), true);
 
         keyManager()->refreshNow(operationContext());
-        auto keyStatus = keyManager()->getKeyForValidation(
+        auto keyStatus = keyManager()->getKeysForValidation(
             operationContext(), 1, LogicalTime(Timestamp(100, 0)));
         ASSERT_EQ(ErrorCodes::KeyNotFound, keyStatus.getStatus());
     }
@@ -346,7 +344,7 @@ TEST_F(KeysManagerShardedTest, ShouldNotCreateKeysWithDisableKeyGenerationFailPo
 
 TEST_F(KeysManagerShardedTest, HasSeenKeysIsFalseUntilKeysAreFound) {
     const LogicalTime currentTime(Timestamp(100, 0));
-    LogicalClock::get(operationContext())->setClusterTimeFromTrustedSource(currentTime);
+    VectorClockMutable::get(operationContext())->tickClusterTimeTo(currentTime);
 
     ASSERT_EQ(false, keyManager()->hasSeenKeys());
 
@@ -356,7 +354,7 @@ TEST_F(KeysManagerShardedTest, HasSeenKeysIsFalseUntilKeysAreFound) {
         keyManager()->enableKeyGenerator(operationContext(), true);
 
         keyManager()->refreshNow(operationContext());
-        auto keyStatus = keyManager()->getKeyForValidation(
+        auto keyStatus = keyManager()->getKeysForValidation(
             operationContext(), 1, LogicalTime(Timestamp(100, 0)));
         ASSERT_EQ(ErrorCodes::KeyNotFound, keyStatus.getStatus());
 
@@ -371,32 +369,180 @@ TEST_F(KeysManagerShardedTest, HasSeenKeysIsFalseUntilKeysAreFound) {
     ASSERT_EQ(true, keyManager()->hasSeenKeys());
 }
 
+class KeysManagerDirectTest : public ConfigServerTestFixture {
+protected:
+    const UUID kMigrationId1 = UUID::gen();
+    const UUID kMigrationId2 = UUID::gen();
+
+    KeysCollectionManager* keyManager() {
+        return _keyManager.get();
+    }
+
+    void setUp() override {
+        ConfigServerTestFixture::setUp();
+
+        auto clockSource = std::make_unique<ClockSourceMock>();
+        // Timestamps of "0 seconds" are not allowed, so we must advance our clock mock to the first
+        // real second.
+        clockSource->advance(Seconds(1));
+
+        operationContext()->getServiceContext()->setFastClockSource(std::move(clockSource));
+        _keyManager = std::make_unique<KeysCollectionManager>(
+            "dummy", std::make_unique<KeysCollectionClientDirect>(), Seconds(1));
+    }
+
+    void tearDown() override {
+        _keyManager->stopMonitoring();
+
+        ConfigServerTestFixture::tearDown();
+    }
+
+private:
+    std::unique_ptr<KeysCollectionManager> _keyManager;
+};
+
+TEST_F(KeysManagerDirectTest, CacheExternalKeyBasic) {
+    keyManager()->startMonitoring(getServiceContext());
+
+    // Refresh immediately to prevent a refresh from discovering the inserted keys.
+    keyManager()->refreshNow(operationContext());
+
+    ExternalKeysCollectionDocument externalKey1(OID::gen(), 1, kMigrationId1);
+    externalKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(100, 0))});
+    ASSERT_OK(insertToConfigCollection(operationContext(),
+                                       NamespaceString::kExternalKeysCollectionNamespace,
+                                       externalKey1.toBSON()));
+
+    keyManager()->cacheExternalKey(externalKey1);
+
+    {
+        auto keyStatus =
+            keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(1, 0)));
+        ASSERT_OK(keyStatus.getStatus());
+        ASSERT_EQ(1, keyStatus.getValue().size());
+
+        auto key = keyStatus.getValue().front();
+        ASSERT_EQ(1, key.getKeyId());
+
+        ASSERT_EQ(externalKey1.getKeyId(), key.getKeyId());
+        ASSERT_EQ(externalKey1.getPurpose(), key.getPurpose());
+    }
+}
+
+TEST_F(KeysManagerDirectTest, WillNotCacheExternalKeyWhenMonitoringIsStopped) {
+    keyManager()->startMonitoring(getServiceContext());
+
+    // Refresh immediately to prevent a refresh from discovering the inserted keys.
+    keyManager()->refreshNow(operationContext());
+
+    // Insert an internal key so the key manager won't attempt to refresh after the refresher is
+    // stopped.
+    KeysCollectionDocument internalKey(1);
+    internalKey.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(100, 0))});
+    ASSERT_OK(insertToConfigCollection(
+        operationContext(), NamespaceString::kKeysCollectionNamespace, internalKey.toBSON()));
+
+    ExternalKeysCollectionDocument externalKey1(OID::gen(), 1, kMigrationId1);
+    externalKey1.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(100, 0))});
+    ASSERT_OK(insertToConfigCollection(operationContext(),
+                                       NamespaceString::kExternalKeysCollectionNamespace,
+                                       externalKey1.toBSON()));
+
+    keyManager()->cacheExternalKey(externalKey1);
+
+    {
+        auto keyStatus =
+            keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(1, 0)));
+        ASSERT_OK(keyStatus.getStatus());
+        ASSERT_EQ(2, keyStatus.getValue().size());
+    }
+
+    keyManager()->stopMonitoring();
+
+    ExternalKeysCollectionDocument externalKey2(OID::gen(), 1, kMigrationId2);
+    externalKey2.setKeysCollectionDocumentBase(
+        {"dummy", TimeProofService::generateRandomKey(), LogicalTime(Timestamp(100, 0))});
+
+    keyManager()->cacheExternalKey(externalKey2);
+
+    // There should still be only the first external key in the cache.
+    {
+        auto keyStatus =
+            keyManager()->getKeysForValidation(operationContext(), 1, LogicalTime(Timestamp(1, 0)));
+        ASSERT_OK(keyStatus.getStatus());
+        ASSERT_EQ(2, keyStatus.getValue().size());
+    }
+}
+
 LogicalTime addSeconds(const LogicalTime& logicalTime, const Seconds& seconds) {
     auto asTimestamp = logicalTime.asTimestamp();
     return LogicalTime(Timestamp(asTimestamp.getSecs() + seconds.count(), asTimestamp.getInc()));
 }
 
-TEST(KeysCollectionManagerUtilTest, HowMuchSleepNeededForCalculationDoesNotOverflow) {
+TEST(KeysCollectionManagerUtilTest, HowMuchSleepNeedForWithDefaultKeysRotationIntervalIs20Days) {
     auto secondsSinceEpoch = durationCount<Seconds>(Date_t::now().toDurationSinceEpoch());
     auto defaultKeysIntervalSeconds = Seconds(KeysRotationIntervalSec);
 
-    // Mock inputs that would have caused an overflow without the changes from SERVER-48709.
-    // "currentTime" is the current logical time in the LogicalClock, which will typically be close
-    // to a timestamp constructed from the number of seconds since the unix epoch. "latestExpiredAt"
-    // is the highest expiration logical time of any key, which will at most be currentTime +
-    // (default key rotation interval * 2) because two valid keys are kept at a time. "interval" is
-    // the duration a key is valid for, which defaults to 90 days = 7,776,000 seconds.
     auto currentTime = LogicalTime(Timestamp(secondsSinceEpoch, 0));
     auto latestExpiredAt = addSeconds(currentTime, defaultKeysIntervalSeconds * 2);
-    auto interval = Milliseconds(defaultKeysIntervalSeconds);
+    auto defaultInterval = Milliseconds(defaultKeysIntervalSeconds);
+
+    auto nextWakeupMillis = keys_collection_manager_util::howMuchSleepNeedFor(
+        currentTime, latestExpiredAt, defaultInterval);
+    ASSERT_EQ(nextWakeupMillis, Days(20));
+}
+
+TEST(KeysCollectionManagerUtilTest, HowMuchSleepNeedForIsNeverLongerThan20Days) {
+    auto secondsSinceEpoch = durationCount<Seconds>(Date_t::now().toDurationSinceEpoch());
+    auto keysRotationInterval = Seconds(Days(50));
+
+    auto currentTime = LogicalTime(Timestamp(secondsSinceEpoch, 0));
+    auto latestExpiredAt = addSeconds(currentTime, keysRotationInterval * 2);
+    auto interval = Milliseconds(keysRotationInterval);
+
+    auto nextWakeupMillis =
+        keys_collection_manager_util::howMuchSleepNeedFor(currentTime, latestExpiredAt, interval);
+    ASSERT_EQ(nextWakeupMillis, Days(20));
+}
+
+TEST(KeysCollectionManagerUtilTest, HowMuchSleepNeedForIsNeverHigherThanRotationInterval) {
+    auto secondsSinceEpoch = durationCount<Seconds>(Date_t::now().toDurationSinceEpoch());
+    auto keysRotationInterval = Seconds(Days(5));
+
+    auto currentTime = LogicalTime(Timestamp(secondsSinceEpoch, 0));
+    auto latestExpiredAt = addSeconds(currentTime, keysRotationInterval * 2);
+    auto interval = Milliseconds(keysRotationInterval);
 
-    // Despite the default rotation interval seconds * 1000 not fitting in a 32 bit unsigned
-    // integer (7,776,000,000 vs. 4,294,967,295), the calculation should not overflow, and the next
-    // wakeup should correctly be the default interval.
     auto nextWakeupMillis =
         keys_collection_manager_util::howMuchSleepNeedFor(currentTime, latestExpiredAt, interval);
     ASSERT_EQ(nextWakeupMillis, interval);
 }
 
+LogicalTime subtractSeconds(const LogicalTime& logicalTime, const Seconds& seconds) {
+    auto asTimestamp = logicalTime.asTimestamp();
+    return LogicalTime(Timestamp(asTimestamp.getSecs() - seconds.count(), asTimestamp.getInc()));
+}
+
+TEST(KeysCollectionManagerUtilTest, HowMuchSleepNeedForAfterNotFindingKeys) {
+    // Default refresh interval if keys could not be found.
+    const Milliseconds kRefreshIntervalIfErrored(200);
+
+    auto secondsSinceEpoch = durationCount<Seconds>(Date_t::now().toDurationSinceEpoch());
+    auto keysRotationInterval = Milliseconds(5000);
+
+    // The latest found key expired before the current time, which means no new keys were found
+    // despite the previous refresh succeeding.
+    auto currentTime = LogicalTime(Timestamp(secondsSinceEpoch, 0));
+    auto latestExpiredAt = subtractSeconds(currentTime, Seconds(1));
+    auto interval = Milliseconds(keysRotationInterval);
+
+    auto nextWakeupMillis =
+        keys_collection_manager_util::howMuchSleepNeedFor(currentTime, latestExpiredAt, interval);
+    ASSERT_EQ(nextWakeupMillis, kRefreshIntervalIfErrored);
+}
+
 }  // namespace
 }  // namespace mongo
