@@ -152,6 +152,31 @@ public:
 
     std::unique_ptr<Pipeline, PipelineDeleter> clone() const;
 
+    /**
+     * Optimize the given pipeline after the stage that 'itr' points to.
+     *
+     * Returns a valid iterator that points to the new "end of the pipeline": i.e., the stage that
+     * comes after 'itr' in the newly optimized pipeline.
+     */
+    static Pipeline::SourceContainer::iterator optimizeEndOfPipeline(
+        Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container);
+
+    /**
+     * Applies optimizeAt() to all stages in the given pipeline after the stage that 'itr' points
+     * to.
+     *
+     * Returns a valid iterator that points to the new "end of the pipeline": i.e., the stage that
+     * comes after 'itr' in the newly optimized pipeline.
+     */
+    static Pipeline::SourceContainer::iterator optimizeAtEndOfPipeline(
+        Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container);
+
+    static std::unique_ptr<Pipeline, PipelineDeleter> makePipelineFromViewDefinition(
+        const boost::intrusive_ptr<ExpressionContext>& subPipelineExpCtx,
+        ExpressionContext::ResolvedNamespace resolvedNs,
+        std::vector<BSONObj> currentPipeline,
+        MakePipelineOptions opts);
+
     const boost::intrusive_ptr<ExpressionContext>& getContext() const {
         return pCtx;
     }
