@@ -335,6 +335,9 @@ void ReplicationCoordinatorImpl::ElectionState::_writeLastVoteForMyElection(
         auto opCtx = cc().makeOperationContext();
         // Any writes that occur as part of an election should not be subject to Flow Control.
         opCtx->setShouldParticipateInFlowControl(false);
+        LOGV2(6015300,
+              "Storing last vote document in local storage for my election",
+              "lastVote"_attr = lastVote);
         return _repl->_externalState->storeLocalLastVoteDocument(opCtx.get(), lastVote);
     }();
 
@@ -345,6 +348,7 @@ void ReplicationCoordinatorImpl::ElectionState::_writeLastVoteForMyElection(
     stdx::lock_guard<Latch> lk(_repl->_mutex);
     LoseElectionDryRunGuardV1 lossGuard(_repl);
     if (status == ErrorCodes::CallbackCanceled) {
+        LOGV2(6015301, "Callback for storing last vote got cancelled");
         return;
     }
 
