@@ -37,6 +37,7 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/active_migrations_registry.h"
 #include "mongo/db/s/chunk_move_write_concern_options.h"
 #include "mongo/db/s/migration_source_manager.h"
@@ -155,20 +156,37 @@ public:
                             tc->setSystemOperationKillableByStepdown(lk);
                         }
                         auto uniqueOpCtx = Client::getCurrent()->makeOperationContext();
-                        auto opCtx = uniqueOpCtx.get();
-                        // Note: This internal authorization is tied to the lifetime of the client.
-                        AuthorizationSession::get(opCtx->getClient())
-                            ->grantInternalAuthorization(opCtx->getClient());
+                        auto executorOpCtx = uniqueOpCtx.get();
 
                         Status status = {ErrorCodes::InternalError, "Uninitialized value"};
 
                         try {
-                            _runImpl(opCtx, moveChunkRequest);
+                            executorOpCtx->setAlwaysInterruptAtStepDownOrUp();
+                            {
+                                // Ensure that opCtx will get interrupted in the event of a
+                                // stepdown. This is to ensure that the MigrationSourceManager
+                                // checks that there are no pending migrationCoordinators
+                                // documents (under the ActiveMigrationRegistry lock) on the
+                                // same term during which the migrationCoordinators document
+                                // will be persisted.
+                                Lock::GlobalLock lk(executorOpCtx, MODE_IX);
+                                uassert(ErrorCodes::InterruptedDueToReplStateChange,
+                                        "Not primary while attempting to start chunk migration "
+                                        "donation",
+                                        repl::ReplicationCoordinator::get(executorOpCtx)
+                                            ->getMemberState()
+                                            .primary());
+                            }
+                            // Note: This internal authorization is tied to the lifetime of the
+                            // client.
+                            AuthorizationSession::get(executorOpCtx->getClient())
+                                ->grantInternalAuthorization(executorOpCtx->getClient());
+                            _runImpl(executorOpCtx, moveChunkRequest);
                             status = Status::OK();
                         } catch (const DBException& e) {
                             status = e.toStatus();
                             if (status.code() == ErrorCodes::LockTimeout) {
-                                ShardingStatistics::get(opCtx)
+                                ShardingStatistics::get(executorOpCtx)
                                     .countDonorMoveChunkLockTimeout.addAndFetch(1);
                             }
                         } catch (const std::exception& e) {
