@@ -165,15 +165,6 @@ void sendAuthenticatedCommandToShards(OperationContext* opCtx,
                                       const BSONObj& command,
                                       const std::vector<ShardId>& shardIds,
                                       const std::shared_ptr<executor::TaskExecutor>& executor) {
-    // TODO SERVER-57519: remove the following scope
-    {
-        // Ensure ShardRegistry is initialized before using the AsyncRequestsSender that relies on
-        // unsafe functions (SERVER-57280)
-        auto shardRegistry = Grid::get(opCtx)->shardRegistry();
-        if (!shardRegistry->isUp()) {
-            shardRegistry->reload(opCtx);
-        }
-    }
 
     // The AsyncRequestsSender ignore impersonation metadata so we need to manually attach them to
     // the command
@@ -496,5 +487,11 @@ void sendDropCollectionParticipantCommandToShards(OperationContext* opCtx,
     }
 }
 
+BSONObj getCriticalSectionReasonForRename(const NamespaceString& from, const NamespaceString& to) {
+    return BSON("command"
+                << "rename"
+                << "from" << from.toString() << "to" << to.toString());
+}
+
 }  // namespace sharding_ddl_util
 }  // namespace mongo
