@@ -37,9 +37,8 @@
 #include "mongo/db/commands/txn_cmds_gen.h"
 #include "mongo/db/commands/txn_two_phase_commit_cmds_gen.h"
 #include "mongo/db/operation_context.h"
-#include "mongo/db/s/wait_for_majority_service.h"
+#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/rpc/metadata/client_metadata.h"
-#include "mongo/rpc/metadata/client_metadata_ismaster.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/unittest/unittest.h"
@@ -67,7 +66,7 @@ void TransactionCoordinatorTestFixture::setUp() {
         shardTargeter->setFindHostReturnValue(makeHostAndPort(shardId));
     }
 
-    WaitForMajorityService::get(getServiceContext()).setUp(getServiceContext());
+    WaitForMajorityService::get(getServiceContext()).startup(getServiceContext());
 }
 
 void TransactionCoordinatorTestFixture::tearDown() {
@@ -75,13 +74,12 @@ void TransactionCoordinatorTestFixture::tearDown() {
     ShardServerTestFixture::tearDown();
 }
 
-std::unique_ptr<ShardingCatalogClient> TransactionCoordinatorTestFixture::makeShardingCatalogClient(
-    std::unique_ptr<DistLockManager> distLockManager) {
+std::unique_ptr<ShardingCatalogClient>
+TransactionCoordinatorTestFixture::makeShardingCatalogClient() {
 
     class StaticCatalogClient final : public ShardingCatalogClientMock {
     public:
-        StaticCatalogClient(std::vector<ShardId> shardIds)
-            : ShardingCatalogClientMock(nullptr), _shardIds(std::move(shardIds)) {}
+        StaticCatalogClient(std::vector<ShardId> shardIds) : _shardIds(std::move(shardIds)) {}
 
         StatusWith<repl::OpTimeWith<std::vector<ShardType>>> getAllShards(
             OperationContext* opCtx, repl::ReadConcernLevel readConcern) override {
@@ -137,7 +135,6 @@ void TransactionCoordinatorTestFixture::associateClientMetadata(Client* client,
                                                &metadataBuilder));
     auto clientMetadata = metadataBuilder.obj();
     auto clientMetadataParse = ClientMetadata::parse(clientMetadata["client"]);
-    ClientMetadataIsMasterState::setClientMetadata(client,
-                                                   std::move(clientMetadataParse.getValue()));
+    ClientMetadata::setAndFinalize(client, std::move(clientMetadataParse.getValue()));
 }
 }  // namespace mongo
