@@ -41,6 +41,14 @@
 namespace mongo {
 namespace {
 
+const NamespaceString kNss("test", "foo");
+
+RoutingTableHistoryValueHandle makeStandaloneRoutingTableHistory(RoutingTableHistory rt) {
+    const auto version = rt.getVersion();
+    return RoutingTableHistoryValueHandle(
+        std::move(rt), ComparableChunkVersion::makeComparableChunkVersion(version));
+}
+
 ChunkRange getRangeForChunk(int i, int nChunks) {
     invariant(i >= 0);
     invariant(nChunks > 0);
@@ -55,24 +63,38 @@ ChunkRange getRangeForChunk(int i, int nChunks) {
 }
 
 template <typename ShardSelectorFn>
-auto makeChunkManagerWithShardSelector(int nShards, uint32_t nChunks, ShardSelectorFn selectShard) {
+CollectionMetadata makeChunkManagerWithShardSelector(int nShards,
+                                                     uint32_t nChunks,
+                                                     ShardSelectorFn selectShard) {
     const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
     const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
 
     std::vector<ChunkType> chunks;
     chunks.reserve(nChunks);
+
     for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
+        chunks.emplace_back(kNss,
                             getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
+                            ChunkVersion{i + 1, 0, collEpoch, boost::none /* timestamp */},
                             selectShard(i, nShards, nChunks));
     }
 
-    auto routingTableHistory = RoutingTableHistory::makeNew(
-        collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-    auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-    return std::make_unique<CollectionMetadata>(std::move(chunkManager), ShardId("shard0"));
+    auto rt = RoutingTableHistory::makeNew(kNss,
+                                           UUID::gen(),
+                                           shardKeyPattern,
+                                           nullptr,
+                                           true,
+                                           collEpoch,
+                                           boost::none /* timestamp */,
+                                           boost::none /* timeseriesFields */,
+                                           boost::none,
+                                           true,
+                                           chunks);
+    return CollectionMetadata(ChunkManager(ShardId("Shard0"),
+                                           DatabaseVersion(UUID::gen()),
+                                           makeStandaloneRoutingTableHistory(std::move(rt)),
+                                           boost::none),
+                              ShardId("shard0"));
 }
 
 ShardId pessimalShardSelector(int i, int nShards, int nChunks) {
@@ -97,32 +119,36 @@ MONGO_COMPILER_NOINLINE auto makeChunkManagerWithOptimalBalancedDistribution(int
 
 MONGO_COMPILER_NOINLINE auto runIncrementalUpdate(const CollectionMetadata& cm,
                                                   const std::vector<ChunkType>& newChunks) {
-    auto rt = cm.getChunkManager()->getRoutingHistory()->makeUpdated(newChunks);
-    return std::make_unique<CollectionMetadata>(std::make_shared<ChunkManager>(rt, boost::none),
-                                                ShardId("shard0"));
+    auto rt = cm.getChunkManager()->getRoutingTableHistory_ForTest().makeUpdated(
+        boost::none, true, newChunks);
+    return CollectionMetadata(ChunkManager(ShardId("shard0"),
+                                           DatabaseVersion(UUID::gen()),
+                                           makeStandaloneRoutingTableHistory(std::move(rt)),
+                                           boost::none),
+                              ShardId("shard0"));
 }
 
 void BM_IncrementalRefreshOfPessimalBalancedDistribution(benchmark::State& state) {
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
-    auto cm = makeChunkManagerWithPessimalBalancedDistribution(nShards, nChunks);
+    auto metadata = makeChunkManagerWithPessimalBalancedDistribution(nShards, nChunks);
 
-    auto postMoveVersion = cm->getChunkManager()->getVersion();
-    const auto collName = NamespaceString(cm->getChunkManager()->getns());
+    auto postMoveVersion = metadata.getChunkManager()->getVersion();
     std::vector<ChunkType> newChunks;
     postMoveVersion.incMajor();
-    newChunks.emplace_back(
-        collName, getRangeForChunk(1, nChunks), postMoveVersion, ShardId("shard0"));
+    newChunks.emplace_back(kNss, getRangeForChunk(1, nChunks), postMoveVersion, ShardId("shard0"));
     postMoveVersion.incMajor();
-    newChunks.emplace_back(
-        collName, getRangeForChunk(3, nChunks), postMoveVersion, ShardId("shard1"));
+    newChunks.emplace_back(kNss, getRangeForChunk(3, nChunks), postMoveVersion, ShardId("shard1"));
 
     for (auto keepRunning : state) {
-        benchmark::DoNotOptimize(runIncrementalUpdate(*cm, newChunks));
+        benchmark::DoNotOptimize(runIncrementalUpdate(metadata, newChunks));
     }
 }
 
-BENCHMARK(BM_IncrementalRefreshOfPessimalBalancedDistribution)->Args({2, 50000});
+BENCHMARK(BM_IncrementalRefreshOfPessimalBalancedDistribution)
+    ->Args({2, 50000})
+    ->Args({2, 250000})
+    ->Args({2, 500000});
 
 template <typename ShardSelectorFn>
 auto BM_FullBuildOfChunkManager(benchmark::State& state, ShardSelectorFn selectShard) {
@@ -130,62 +156,36 @@ auto BM_FullBuildOfChunkManager(benchmark::State& state, ShardSelectorFn selectS
     const uint32_t nChunks = state.range(1);
 
     const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
     const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
 
     std::vector<ChunkType> chunks;
     chunks.reserve(nChunks);
+
     for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
+        chunks.emplace_back(kNss,
                             getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
+                            ChunkVersion{i + 1, 0, collEpoch, boost::none /* timestamp */},
                             selectShard(i, nShards, nChunks));
     }
 
     for (auto keepRunning : state) {
-        auto routingTableHistory = RoutingTableHistory::makeNew(
-            collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-        auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-        benchmark::DoNotOptimize(CollectionMetadata(std::move(chunkManager), ShardId("shard0")));
-    }
-}
-
-template <typename ShardSelectorFn>
-auto BM_GetNextOrphanedRange(benchmark::State& state, ShardSelectorFn selectShard) {
-    const int nShards = state.range(0);
-    const uint32_t nChunks = state.range(1);
-
-    const auto collEpoch = OID::gen();
-    const auto collName = NamespaceString("test.foo");
-    const auto shardKeyPattern = KeyPattern(BSON("_id" << 1));
-
-    std::vector<ChunkType> chunks;
-    chunks.reserve(nChunks);
-    for (uint32_t i = 0; i < nChunks; ++i) {
-        chunks.emplace_back(collName,
-                            getRangeForChunk(i, nChunks),
-                            ChunkVersion{i + 1, 0, collEpoch},
-                            selectShard(i, nShards, nChunks));
-    }
-    auto routingTableHistory = RoutingTableHistory::makeNew(
-        collName, UUID::gen(), shardKeyPattern, nullptr, true, collEpoch, chunks);
-    auto chunkManager = std::make_shared<ChunkManager>(routingTableHistory, boost::none);
-    CollectionMetadata metadata(std::move(chunkManager), ShardId("shard0"));
-    const auto emptyReceivingChunks =
-        RangeMap{SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<BSONObj>()};
-
-    for (auto keepRunning : state) {
-        auto startingKey = metadata.getMinKey();
-        auto chunkMap = metadata.getOwnedChunks();
-        while (true) {
-            auto range = metadata.getNextOrphanRange(chunkMap, emptyReceivingChunks, startingKey);
-            if (!range) {
-                break;
-            }
-
-            startingKey = range->getMax();
-            benchmark::DoNotOptimize(range);
-        }
+        auto rt = RoutingTableHistory::makeNew(kNss,
+                                               UUID::gen(),
+                                               shardKeyPattern,
+                                               nullptr,
+                                               true,
+                                               collEpoch,
+                                               boost::none /* timestamp */,
+                                               boost::none /* timeseriesFields */,
+                                               boost::none,
+                                               true,
+                                               chunks);
+        benchmark::DoNotOptimize(
+            CollectionMetadata(ChunkManager(ShardId("shard0"),
+                                            DatabaseVersion(UUID::gen()),
+                                            makeStandaloneRoutingTableHistory(std::move(rt)),
+                                            boost::none),
+                               ShardId("shard0")));
     }
 }
 
@@ -273,13 +273,13 @@ void BM_FindIntersectingChunk(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto keysIter = makeCircularIterator(keys);
 
     for (auto keepRunning : state) {
         benchmark::DoNotOptimize(
-            cm->getChunkManager()->findIntersectingChunkWithSimpleCollation(*keysIter));
+            metadata.getChunkManager()->findIntersectingChunkWithSimpleCollation(*keysIter));
         ++keysIter;
     }
 
@@ -292,14 +292,14 @@ void BM_GetShardIdsForRange(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto ranges = makeRanges(keys);
     auto rangesIter = makeCircularIterator(ranges);
 
     for (auto keepRunning : state) {
         std::set<ShardId> shardIds;
-        cm->getChunkManager()->getShardIdsForRange(
+        metadata.getChunkManager()->getShardIdsForRange(
             rangesIter->first, rangesIter->second, &shardIds);
         ++rangesIter;
     }
@@ -313,13 +313,13 @@ void BM_GetShardIdsForRangeMinKeyToMaxKey(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto min = BSON("_id" << MINKEY);
     auto max = BSON("_id" << MAXKEY);
 
     for (auto keepRunning : state) {
         std::set<ShardId> shardIds;
-        cm->getChunkManager()->getShardIdsForRange(min, max, &shardIds);
+        metadata.getChunkManager()->getShardIdsForRange(min, max, &shardIds);
     }
 
     state.SetItemsProcessed(state.iterations());
@@ -331,14 +331,14 @@ void BM_KeyBelongsToMe(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto keysIter = makeCircularIterator(keys);
 
     size_t nOwned = 0;
 
     for (auto keepRunning : state) {
-        if (cm->keyBelongsToMe(*keysIter)) {
+        if (metadata.keyBelongsToMe(*keysIter)) {
             ++nOwned;
         }
         ++keysIter;
@@ -354,7 +354,7 @@ void BM_RangeOverlapsChunk(benchmark::State& state,
     const int nShards = state.range(0);
     const int nChunks = state.range(1);
 
-    auto cm = makeCollectionMetadata(nShards, nChunks);
+    auto metadata = makeCollectionMetadata(nShards, nChunks);
     auto keys = makeKeys(nChunks);
     auto ranges = makeRanges(keys);
     auto rangesIter = makeCircularIterator(ranges);
@@ -362,7 +362,7 @@ void BM_RangeOverlapsChunk(benchmark::State& state,
     size_t nOverlapped = 0;
 
     for (auto keepRunning : state) {
-        if (cm->rangeOverlapsChunk(ChunkRange(rangesIter->first, rangesIter->second))) {
+        if (metadata.rangeOverlapsChunk(ChunkRange(rangesIter->first, rangesIter->second))) {
             ++nOverlapped;
         }
         ++rangesIter;
@@ -382,8 +382,6 @@ MONGO_INITIALIZER(RegisterBenchmarks)(InitializerContext* context) {
     std::initializer_list<benchmark::internal::Benchmark*> bmCases{
         REGISTER_BENCHMARK_CAPTURE(BM_FullBuildOfChunkManager, Pessimal, pessimalShardSelector),
         REGISTER_BENCHMARK_CAPTURE(BM_FullBuildOfChunkManager, Optimal, optimalShardSelector),
-        REGISTER_BENCHMARK_CAPTURE(BM_GetNextOrphanedRange, Pessimal, pessimalShardSelector),
-        REGISTER_BENCHMARK_CAPTURE(BM_GetNextOrphanedRange, Optimal, optimalShardSelector),
         REGISTER_BENCHMARK_CAPTURE(
             BM_FindIntersectingChunk, Pessimal, makeChunkManagerWithPessimalBalancedDistribution),
         REGISTER_BENCHMARK_CAPTURE(
@@ -415,8 +413,6 @@ MONGO_INITIALIZER(RegisterBenchmarks)(InitializerContext* context) {
             ->Args({1000, 50000})
             ->Args({2, 2});
     }
-
-    return Status::OK();
 }
 
 }  // namespace
