@@ -46,6 +46,7 @@ DBQuery.prototype.help = function() {
     print("\t.allowPartialResults()");
     print("\t.returnKey()");
     print("\t.showRecordId() - adds a $recordId field to each returned object");
+    print("\t.allowDiskUse() - allow using disk in completing the query");
 
     print("\nCursor methods");
     print("\t.toArray() - iterates through docs and returns an array of the results");
@@ -57,6 +58,7 @@ DBQuery.prototype.help = function() {
     print(
         "\t.objsLeftInBatch() - returns count of docs left in current batch (when exhausted, a new getMore will be issued)");
     print("\t.itcount() - iterates through documents and counts them");
+    print("\t.getClusterTime() - returns the read timestamp for snapshot reads");
     print("\t.pretty() - pretty print each document, possibly over multiple lines");
 };
 
@@ -126,6 +128,10 @@ DBQuery.prototype._exec = function() {
                 throw new Error("collation requires use of read commands");
             }
 
+            if (this._special && this._query._allowDiskUse) {
+                throw new Error("allowDiskUse option requires use of read commands");
+            }
+
             this._cursor = this._mongo.find(this._ns,
                                             this._query,
                                             this._fields,
@@ -224,12 +230,12 @@ DBQuery.prototype._convertToCommand = function(canAttachReadPref) {
         cmd["collation"] = this._query.collation;
     }
 
-    if ((this._options & DBQuery.Option.tailable) != 0) {
-        cmd["tailable"] = true;
+    if ("allowDiskUse" in this._query) {
+        cmd["allowDiskUse"] = this._query.allowDiskUse;
     }
 
-    if ((this._options & DBQuery.Option.oplogReplay) != 0) {
-        cmd["oplogReplay"] = true;
+    if ((this._options & DBQuery.Option.tailable) != 0) {
+        cmd["tailable"] = true;
     }
 
     if ((this._options & DBQuery.Option.noTimeout) != 0) {
@@ -323,6 +329,11 @@ DBQuery.prototype.readOnly = function() {
     return this;
 };
 
+DBQuery.prototype.getId = function() {
+    this._exec();
+    return this._cursor.getId();
+};
+
 DBQuery.prototype.toArray = function() {
     if (this._arr)
         return this._arr;
@@ -460,8 +471,9 @@ DBQuery.prototype.maxTimeMS = function(maxTimeMS) {
     return this._addSpecial("$maxTimeMS", maxTimeMS);
 };
 
-DBQuery.prototype.readConcern = function(level) {
-    var readConcernObj = {level: level};
+DBQuery.prototype.readConcern = function(level, atClusterTime = undefined) {
+    var readConcernObj =
+        atClusterTime ? {level: level, atClusterTime: atClusterTime} : {level: level};
 
     return this._addSpecial("readConcern", readConcernObj);
 };
@@ -470,22 +482,30 @@ DBQuery.prototype.collation = function(collationSpec) {
     return this._addSpecial("collation", collationSpec);
 };
 
+DBQuery.prototype.allowDiskUse = function() {
+    return this._addSpecial("allowDiskUse", true);
+};
+
 /**
  * Sets the read preference for this cursor.
  *
  * @param mode {string} read preference mode to use.
  * @param tagSet {Array.<Object>} optional. The list of tags to use, order matters.
- *     Note that this object only keeps a shallow copy of this array.
+ * @param hedgeOptions {<Object>} optional. The hedge options of the form {enabled: <bool>}.
  *
  * @return this cursor
  */
-DBQuery.prototype.readPref = function(mode, tagSet) {
+DBQuery.prototype.readPref = function(mode, tagSet, hedgeOptions) {
     var readPrefObj = {mode: mode};
 
     if (tagSet) {
         readPrefObj.tags = tagSet;
     }
 
+    if (hedgeOptions) {
+        readPrefObj.hedge = hedgeOptions;
+    }
+
     return this._addSpecial("$readPreference", readPrefObj);
 };
 
@@ -582,19 +602,6 @@ DBQuery.prototype.noCursorTimeout = function() {
     return this;
 };
 
-/**
- * Internal replication use only - driver should not set
- *
- * @method
- * @see http://docs.mongodb.org/meta-driver/latest/legacy/mongodb-wire-protocol/#op-query
- * @return {DBQuery}
- */
-DBQuery.prototype.oplogReplay = function() {
-    this._checkModify();
-    this.addOption(DBQuery.Option.oplogReplay);
-    return this;
-};
-
 /**
  * Limits the fields to return for all matching documents.
  *
@@ -670,6 +677,12 @@ DBQuery.prototype.isExhausted = function() {
     return this._cursor.isClosed() && this._cursor.objsLeftInBatch() === 0;
 };
 
+DBQuery.prototype.getClusterTime = function() {
+    // Return the read timestamp for snapshot reads, or undefined for other readConcern levels.
+    this._exec();
+    return this._cursor.getClusterTime();
+};
+
 DBQuery.shellBatchSize = 20;
 
 /**
@@ -679,7 +692,9 @@ DBQuery.shellBatchSize = 20;
 DBQuery.Option = {
     tailable: 0x2,
     slaveOk: 0x4,
-    oplogReplay: 0x8,
+    // 0x8 is reserved for oplogReplay, but not explicitly defined. This is because the flag no
+    // longer has any meaning to the server, and will be ignored, so there is no reason for it to
+    // be set by clients.
     noTimeout: 0x10,
     awaitData: 0x20,
     exhaust: 0x40,
@@ -698,6 +713,12 @@ function DBCommandCursor(db, cmdResult, batchSize, maxAwaitTimeMS, txnNumber) {
 
     this._batch = cmdResult.cursor.firstBatch.reverse();  // modifies input to allow popping
 
+    // If the command result represents a snapshot read cursor, update our atClusterTime. And this
+    // atClusterTime should not change over the lifetime of the cursor.
+    if (cmdResult.cursor.atClusterTime) {
+        this._atClusterTime = cmdResult.cursor.atClusterTime;
+    }
+
     // If the command result represents a change stream cursor, update our postBatchResumeToken.
     this._updatePostBatchResumeToken(cmdResult.cursor);
 
@@ -819,6 +840,12 @@ DBCommandCursor.prototype._runGetMoreCommand = function() {
 
     // Successfully retrieved the next batch.
     this._batch = cmdRes.cursor.nextBatch.reverse();
+
+    // The read timestamp of a snapshot read cursor should not change over the lifetime of the
+    // cursor.
+    if (cmdRes.cursor.atClusterTime) {
+        assert.eq(this._atClusterTime, cmdRes.cursor.atClusterTime);
+    }
 };
 
 DBCommandCursor.prototype._hasNextUsingCommands = function() {
@@ -878,11 +905,19 @@ DBCommandCursor.prototype.objsLeftInBatch = function() {
         return this._cursor.objsLeftInBatch();
     }
 };
+DBCommandCursor.prototype.getId = function() {
+    return this._cursorid;
+};
 DBCommandCursor.prototype.getResumeToken = function() {
     // Return the most recent recorded resume token, if such a token exists.
     return this._resumeToken;
 };
 
+DBCommandCursor.prototype.getClusterTime = function() {
+    // Return the read timestamp for snapshot reads, or undefined for other readConcern levels.
+    return this._atClusterTime;
+};
+
 DBCommandCursor.prototype.help = function() {
     // This is the same as the "Cursor Methods" section of DBQuery.help().
     print("\nCursor methods");
@@ -896,6 +931,7 @@ DBCommandCursor.prototype.help = function() {
     print("\t.itcount() - iterates through documents and counts them");
     print(
         "\t.getResumeToken() - for a change stream cursor, obtains the most recent valid resume token, if it exists.");
+    print("\t.getClusterTime() - returns the read timestamp for snapshot reads.");
     print("\t.pretty() - pretty print each document, possibly over multiple lines");
     print("\t.close()");
 };
